// Adds a horizontal line guide
function webdeveloper_addHorizontalLineGuide()
{
    var color              = webdeveloper_getStringPreference("webdeveloper.line.guides.color");
    var contentDocument    = getBrowser().contentDocument;
    var contentWindow      = getBrowser().contentWindow;
    var documentHeight     = null;
    var documentList       = webdeveloper_getDocuments(webdeveloper_getContentWindow());
    var documentLength     = documentList.length;
    var lineGuide          = null;
    var lineGuidePositions = null;
    var offset             = contentWindow.pageYOffset;
    var pageDocument       = null;
    var spacing            = 0;

    // Loop through the documents
    for(var i = 0; i < documentLength; i++)
    {
        pageDocument                    = documentList[i];
        documentHeight                  = pageDocument.height;
        lineGuide                       = pageDocument.createElement("div");
        lineGuide.style.backgroundColor = color;
        lineGuide.style.top             = 0;
        lineGuidePositions              = webdeveloper_getHorizontalLineGuidePositions(pageDocument);
        spacing                         = 100;

        // If this is the content document
        if(pageDocument == contentDocument)
        {
            spacing += offset;
        }

        lineGuide.addEventListener("mousedown", webdeveloper_selectLineGuide, true);
        lineGuide.addEventListener("mouseout", webdeveloper_mouseoutLineGuide, true);
        lineGuide.addEventListener("mouseover", webdeveloper_mouseoverLineGuide, true);
        lineGuide.addEventListener("mouseup", webdeveloper_deselectLineGuide, true);

        webdeveloper_sizeHorizontalLineGuide(lineGuide, pageDocument);

        // While the spacing is less than the document height
        while(spacing < documentHeight)
        {
            // If there is already a line guide at this position
            if(webdeveloper_contains(lineGuidePositions, spacing + "px"))
            {
                spacing += 100;
            }
            else
            {
                lineGuide.style.top = spacing + "px";
                break;
            }
        }

        lineGuide.setAttribute("class", "webdeveloper-line-guide webdeveloper-horizontal-line-guide");
        pageDocument.body.appendChild(lineGuide);
    }
}

// Adds a vertical line guide
function webdeveloper_addVerticalLineGuide()
{
    var color              = webdeveloper_getStringPreference("webdeveloper.line.guides.color");
    var contentDocument    = getBrowser().contentDocument;
    var contentWindow      = getBrowser().contentWindow;
    var documentList       = webdeveloper_getDocuments(webdeveloper_getContentWindow());
    var documentLength     = documentList.length;
    var documentWidth      = null;
    var lineGuide          = null;
    var lineGuidePositions = null;
    var offset             = contentWindow.pageXOffset;
    var pageDocument       = null;
    var spacing            = 0;

    // Loop through the documents
    for(var i = 0; i < documentLength; i++)
    {
        pageDocument                    = documentList[i];
        documentWidth                   = pageDocument.width;
        lineGuide                       = pageDocument.createElement("div");
        lineGuide.style.backgroundColor = color;
        lineGuide.style.left            = 0;
        lineGuidePositions              = webdeveloper_getVerticalLineGuidePositions(pageDocument);
        spacing                         = 100;

        // If this is the content document
        if(pageDocument == contentDocument)
        {
            spacing += offset;
        }

        lineGuide.addEventListener("mousedown", webdeveloper_selectLineGuide, true);
        lineGuide.addEventListener("mouseout", webdeveloper_mouseoutLineGuide, true);
        lineGuide.addEventListener("mouseover", webdeveloper_mouseoverLineGuide, true);
        lineGuide.addEventListener("mouseup", webdeveloper_deselectLineGuide, true);

        webdeveloper_sizeVerticalLineGuide(lineGuide, pageDocument);

        // While the spacing is less than the document width
        while(spacing < documentWidth)
        {
            // If there is already a line guide at this position
            if(webdeveloper_contains(lineGuidePositions, spacing + "px"))
            {
                spacing += 100;
            }
            else
            {
                lineGuide.style.left = spacing + "px";
                break;
            }
        }

        lineGuide.setAttribute("class", "webdeveloper-line-guide webdeveloper-vertical-line-guide");
        pageDocument.body.appendChild(lineGuide);
    }
}

// Called when a line guide is deselected
function webdeveloper_deselectLineGuide(event)
{
    webdeveloper_lineGuideSelected = null;
}

// Displays line guides
function webdeveloper_displayLineGuides(element, applyStyle)
{
    var checked        = element.getAttribute("checked");
    var divElement     = null;
    var documentList   = webdeveloper_getDocuments(webdeveloper_getContentWindow());
    var documentLength = documentList.length;
    var pageDocument   = null;

    webdeveloper_configureElement(document.getElementById("webdeveloper-line-guides-toolbar"), "hidden", !checked);

    // Loop through the documents
    for(var i = 0; i < documentLength; i++)
    {
        pageDocument = documentList[i];

        // If displaying line guides
        if(checked)
        {
            divElement = pageDocument.createElement("div");

            divElement.setAttribute("id", "webdeveloper-line-guide-information");
            pageDocument.addEventListener("mousemove", webdeveloper_moveLineGuide, true);
            pageDocument.body.appendChild(divElement);
        }
        else
        {
            divElement = pageDocument.getElementById("webdeveloper-line-guide-information");

            webdeveloper_removeLineGuides(pageDocument);

            // If the div element is set
            if(divElement)
            {
                divElement.parentNode.removeChild(divElement);
            }

            // Try to remove the event listener
            try
            {
                pageDocument.removeEventListener("mousemove", webdeveloper_moveLineGuide, true);
            }
            catch(exception)
            {
                // Do nothing
            }
        }
    }

    // If displaying line guides
    if(checked)
    {
        document.getElementById("webdeveloper.line.guides.color").color = webdeveloper_getStringPreference("webdeveloper.line.guides.color");

        webdeveloper_addHorizontalLineGuide();
        webdeveloper_addVerticalLineGuide();
        window.addEventListener("resize", webdeveloper_resizeLineGuides, true);
    }
    else
    {
        // Try to remove the event listener
        try
        {
            window.removeEventListener("resize", webdeveloper_resizeLineGuides, true);
        }
        catch(exception)
        {
            // Do nothing
        }

        webdeveloper_lineGuideSelected = null;
    }

    webdeveloper_toggleStyleSheet(element, "chrome://webdeveloper/content/stylesheets/display_line_guides.css", "webdeveloper-display-line-guides", applyStyle);
}

// Returns an array containing the horizontal line guide positions
function webdeveloper_getHorizontalLineGuidePositions(pageDocument)
{
    return webdeveloper_getLineGuidePositions(pageDocument, true);
}

// Returns the line guide position nearest to the given line guide position
function webdeveloper_getLineGuidePosition(pageDocument, horizontal, lineGuidePosition, next)
{
    var lineGuidePositions     = webdeveloper_getLineGuidePositions(pageDocument, horizontal);
    var lineGuidesLength       = lineGuidePositions.length;
    var otherLineGuidePosition = 0;
    var position               = 0;

    // Loop through the line guide positions
    for(var i = 0; i < lineGuidesLength; i++)
    {
        otherLineGuidePosition = lineGuidePositions[i];
        otherLineGuidePosition = otherLineGuidePosition.substring(0, otherLineGuidePosition.length - 2);

        // If looking for the next line guide position, the other line guide position is greater than the line guide position and the other line guide position is greater than the saved position
        if(next && otherLineGuidePosition > lineGuidePosition && otherLineGuidePosition > position)
        {
            position = otherLineGuidePosition;
        }
        else if(!next && otherLineGuidePosition < lineGuidePosition && otherLineGuidePosition > position)
        {
            position = otherLineGuidePosition;
        }
    }

    return position;
}

// Returns an array containing the line guide positions
function webdeveloper_getLineGuidePositions(pageDocument, horizontal)
{
    var divElement         = null;
    var divElementClass    = null;
    var divElements        = pageDocument.getElementsByTagName("div");
    var divElementsLength  = divElements.length;
    var lineGuidePositions = new Array();

    // Loop through the div elements
    for(var i = 0; i < divElementsLength; i++)
    {
        divElement = divElements[i];

        // If the div element is set
        if(divElement)
        {
            divElementClass = divElement.getAttribute("class");

            // If the class exists and contains webdeveloper-line-guide
            if(divElementClass && divElementClass.indexOf("webdeveloper-line-guide") != -1)
            {
                // If looking for horizontal line guides and the class contains webdeveloper-horizontal-line-guide
                if(horizontal && divElementClass.indexOf("webdeveloper-horizontal-line-guide") != -1)
                {
                    lineGuidePositions.push(divElement.style.top);
                }
                else if(!horizontal && divElementClass.indexOf("webdeveloper-vertical-line-guide") != -1)
                {
                    lineGuidePositions.push(divElement.style.left);
                }
            }
        }
    }

    return lineGuidePositions;
}

// Returns an array containing the vertical line guide positions
function webdeveloper_getVerticalLineGuidePositions(pageDocument)
{
    return webdeveloper_getLineGuidePositions(pageDocument, false);
}

// Called when the mouse is not over a line guide
function webdeveloper_mouseoutLineGuide(event)
{
    var element = event.target;

    // If the element is set
    if(element)
    {
        var ownerDocument = element.ownerDocument;

        // If the element has an owner document
        if(ownerDocument)
        {
            var lineGuideInformation = ownerDocument.getElementById("webdeveloper-line-guide-information");

            // If the line guide information is found
            if(lineGuideInformation)
            {
                lineGuideInformation.style.display = "none";
            }
        }
    }
}

// Called when the mouse is over a line guide
function webdeveloper_mouseoverLineGuide(event)
{
    var element = event.target;

    // If the element is set
    if(element)
    {
        var ownerDocument = element.ownerDocument;

        // If the element has an owner document
        if(ownerDocument)
        {
            var lineGuideInformation = ownerDocument.getElementById("webdeveloper-line-guide-information");

            // If the line guide information is found
            if(lineGuideInformation)
            {
                var headerElement       = ownerDocument.createElement("h1");
                var horizontalLineGuide = true;
                var lineGuidePositions  = null;
                var pElement            = ownerDocument.createElement("p");
                var position            = 0;
                var stringBundle        = document.getElementById("webdeveloper-string-bundle");
                var xPosition           = event.pageX;
                var yPosition           = event.pageY;

                // If the line guide has a class attribute and it contains webdeveloper-vertical-line-guide
                if(element.hasAttribute("class") && element.getAttribute("class").indexOf("webdeveloper-vertical-line-guide") != -1)
                {
                    horizontalLineGuide = false;
                }

                // If this is a horizontal line guide
                if(horizontalLineGuide)
                {
                    position = yPosition;
                }
                else
                {
                    position = xPosition;
                }

                webdeveloper_removeAllChildNodes(lineGuideInformation);

                headerElement.appendChild(ownerDocument.createTextNode(stringBundle.getString("webdeveloper_position") + " = " + position));
                lineGuideInformation.appendChild(headerElement);

                pElement.appendChild(ownerDocument.createTextNode(stringBundle.getString("webdeveloper_previousLineGuide") + " = " + webdeveloper_getLineGuidePosition(ownerDocument, horizontalLineGuide, position, false)));
                lineGuideInformation.appendChild(pElement);

                pElement = ownerDocument.createElement("p");
                pElement.appendChild(ownerDocument.createTextNode(stringBundle.getString("webdeveloper_nextLineGuide") + " = " + webdeveloper_getLineGuidePosition(ownerDocument, horizontalLineGuide, position, true)));
                lineGuideInformation.appendChild(pElement);

                webdeveloper_adjustElementPosition(lineGuideInformation, xPosition, yPosition, 10);

                // Show the line guide information
                lineGuideInformation.style.display = "block";
            }
        }
    }
}

// Called when a line guide is moved
function webdeveloper_moveLineGuide(event)
{
    // If a line guide is selected
    if(webdeveloper_lineGuideSelected)
    {
        // If the line guide has a class attribute and it contains webdeveloper-horizontal-line-guide
        if(webdeveloper_lineGuideSelected.hasAttribute("class") && webdeveloper_lineGuideSelected.getAttribute("class").indexOf("webdeveloper-horizontal-line-guide") != -1)
        {
            webdeveloper_lineGuideSelected.style.top = event.pageY + "px";
        }
        else
        {
            webdeveloper_lineGuideSelected.style.left = event.pageX + "px";
        }
    }
}

// Remove the line guides from the document
function webdeveloper_removeLineGuides(ownerDocument)
{
    var divElement        = null;
    var divElements       = ownerDocument.getElementsByTagName("div");
    var divElementsLength = divElements.length;

    // Loop through the div elements
    for(var i = 0; i < divElementsLength; i++)
    {
        divElement = divElements[i];

        // If the div element is set and the class exists and contains webdeveloper-line-guide
        if(divElement && divElement.hasAttribute("class") && divElement.getAttribute("class").indexOf("webdeveloper-line-guide") != -1)
        {
            divElement.parentNode.removeChild(divElement);
            i--;
        }
    }
}

// Resizes the line guides when the window is resized
function webdeveloper_resizeLineGuides(event)
{
    var divElement        = null;
    var divElementClass   = null;
    var divElements       = null;
    var divElementsLength = null;
    var documentList      = webdeveloper_getDocuments(webdeveloper_getContentWindow());
    var documentLength    = documentList.length;
    var pageDocument      = null;

    // Loop through the documents
    for(var i = 0; i < documentLength; i++)
    {
        pageDocument      = documentList[i];
        divElements       = pageDocument.getElementsByTagName("div");
        divElementsLength = divElements.length;

        // Loop through the div elements
        for(var j = 0; j < divElementsLength; j++)
        {
            divElement = divElements[j];

            // If the div element is set
            if(divElement)
            {
                divElementClass = divElement.getAttribute("class");

                // If the class exists and contains webdeveloper-line-guide
                if(divElementClass && divElementClass.indexOf("webdeveloper-line-guide") != -1)
                {
                    // If the class contains webdeveloper-horizontal-line-guide
                    if(divElementClass.indexOf("webdeveloper-horizontal-line-guide") != -1)
                    {
                        webdeveloper_sizeHorizontalLineGuide(divElement, pageDocument);
                    }
                    else
                    {
                        webdeveloper_sizeVerticalLineGuide(divElement, pageDocument);
                    }
                }
            }
        }
    }

}

// Called when a line guide is selected
function webdeveloper_selectLineGuide(event)
{
    // If the click was not a right click
    if(event.button != 2)
    {
        var element = event.target;

        // If the element is set
        if(element)
        {
            webdeveloper_lineGuideSelected = element;
        }
    }
}

// Sets the size of a horizontal line guide
function webdeveloper_sizeHorizontalLineGuide(lineGuide, pageDocument)
{
    webdeveloper_sizeLineGuide(lineGuide, pageDocument, true);
}

// Sets the size of a line guide
function webdeveloper_sizeLineGuide(lineGuide, pageDocument, horizontal)
{
    // If the line guide and page document are set
    if(lineGuide && pageDocument)
    {
        var contentWindow = getBrowser().contentWindow;

        // If sizing a horizontal line guide
        if(horizontal)
        {
            var documentWidth = pageDocument.width;
            var viewportWidth = contentWindow.innerWidth;

            // If the viewport width is greater than the document width
            if(viewportWidth > documentWidth)
            {
                lineGuide.style.width = viewportWidth + "px";
            }
            else
            {
                lineGuide.style.width = documentWidth + "px";
            }
        }
        else
        {
            var documentHeight = pageDocument.height;
            var viewportHeight = contentWindow.innerHeight;

            // If the viewport height is greater than the document height
            if(viewportHeight > documentHeight)
            {
                lineGuide.style.height = viewportHeight + "px";
            }
            else
            {
                lineGuide.style.height = documentHeight + "px";
            }
        }
    }
}

// Sets the size of a vertical line guide
function webdeveloper_sizeVerticalLineGuide(lineGuide, pageDocument)
{
    webdeveloper_sizeLineGuide(lineGuide, pageDocument, false);
}

// Updates the line guides color
function webdeveloper_updateLineGuideColor(colorPicker)
{
    // If the color picker is set
    if(colorPicker)
    {
        var color             = colorPicker.color;
        var divElement        = null;
        var divElements       = null;
        var divElementsLength = null;
        var documentList      = webdeveloper_getDocuments(webdeveloper_getContentWindow());
        var documentLength    = documentList.length;

        webdeveloper_setStringPreference("webdeveloper.line.guides.color", color);

        // Loop through the documents
        for(var i = 0; i < documentLength; i++)
        {
            divElements       = documentList[i].getElementsByTagName("div");
            divElementsLength = divElements.length;

            // Loop through the div elements
            for(var j = 0; j < divElementsLength; j++)
            {
                divElement = divElements[j];

                // If the div element is set and the class exists and contains webdeveloper-line-guide
                if(divElement && divElement.hasAttribute("class") && divElement.getAttribute("class").indexOf("webdeveloper-line-guide") != -1)
                {
                    divElement.style.backgroundColor = color;
                }
            }
        }
    }
}
